@@ -428,7 +428,10 @@ MMAlignResult WrappedMMAlign(const std::vector<geom::Vec3List>& pos_one,
                              const ost::seq::SequenceList& seq2,
                              const std::vector<bool>& rna1,
                              const std::vector<bool>& rna2,
-                             bool fast) {
+                             bool fast,
+                             const std::map<int, int>& mapping) {
+
+  std::map<int, int> chainmap(mapping);
 
   // input checks
   if(pos_one.empty() || pos_two.empty()) {
@@ -722,6 +725,11 @@ MMAlignResult WrappedMMAlign(const std::vector<geom::Vec3List>& pos_one,
                 TMave_mat[i][j]=-1;
                 continue;
             }
+            if (chainmap.size() && (!chainmap.count(i) || chainmap[i]!=j))
+            {
+                TMave_mat[i][j]=-1;
+                continue;
+            }
 
             ylen=ylen_vec[j];
             if (ylen<3)
@@ -755,8 +763,8 @@ MMAlignResult WrappedMMAlign(const std::vector<geom::Vec3List>& pos_one,
             
             if (byresi_opt)
             {
-                int total_aln=extract_aln_from_resi(sequence,
-                    seqx,seqy,resi_vec1,resi_vec2,xlen_vec,ylen_vec, i, j);
+                int total_aln=extract_aln_from_resi(sequence, seqx,seqy,
+                    resi_vec1,resi_vec2,xlen_vec,ylen_vec, i, j, byresi_opt);
                 seqxA_mat[i][j]=sequence[0];
                 seqyA_mat[i][j]=sequence[1];
                 if (total_aln>xlen+ylen-3)
@@ -827,7 +835,7 @@ MMAlignResult WrappedMMAlign(const std::vector<geom::Vec3List>& pos_one,
     /* refine alignment for large oligomers */
     int aln_chain_num=count_assign_pair(assign1_list,chain1_num);
     bool is_oligomer=(aln_chain_num>=3);
-    if (aln_chain_num==2) // dimer alignment
+    if (aln_chain_num==2 && chainmap.size()==0) // dimer alignment
     {
         int na_chain_num1,na_chain_num2,aa_chain_num1,aa_chain_num2;
         count_na_aa_chain_num(na_chain_num1,aa_chain_num1,mol_vec1);
@@ -849,7 +857,7 @@ MMAlignResult WrappedMMAlign(const std::vector<geom::Vec3List>& pos_one,
         else is_oligomer=true; /* align oligomers to dimer */
     }
 
-    if (aln_chain_num>=3 || is_oligomer) // oligomer alignment
+    if ((aln_chain_num>=3 || is_oligomer) && chainmap.size()==0) // oligomer alignment
     {
         /* extract centroid coordinates */
         double **xcentroids;
@@ -868,7 +876,7 @@ MMAlignResult WrappedMMAlign(const std::vector<geom::Vec3List>& pos_one,
         hetero_refined_greedy_search(TMave_mat, assign1_list,
             assign2_list, chain1_num, chain2_num, xcentroids,
             ycentroids, d0MM, len_aa+len_na);
-        
+
         /* clean up */
         DeleteArray(&xcentroids, chain1_num);
         DeleteArray(&ycentroids, chain2_num);
@@ -897,7 +905,45 @@ MMAlignResult WrappedMMAlign(const std::vector<geom::Vec3List>& pos_one,
         seqx_vec, seqy_vec, secx_vec, secy_vec, mol_vec1, mol_vec2, xlen_vec,
         ylen_vec, xa, ya, seqx, seqy, secx, secy, len_aa, len_na, chain1_num,
         chain2_num, TMave_mat, seqxA_mat, seqyA_mat, assign1_list, assign2_list,
-        sequence, d0_scale, fast_opt);
+        sequence, d0_scale, fast_opt, chainmap);
+    
+    if (byresi_opt && aln_chain_num>=4 && is_oligomer && chainmap.size()==0) // oligomer alignment
+    {
+        MMalign_final(xname.substr(dir1_opt.size()), yname.substr(dir2_opt.size()),
+            chainID_list1, chainID_list2,
+            fname_super, fname_lign, fname_matrix,
+            xa_vec, ya_vec, seqx_vec, seqy_vec,
+            secx_vec, secy_vec, mol_vec1, mol_vec2, xlen_vec, ylen_vec,
+            xa, ya, seqx, seqy, secx, secy, len_aa, len_na,
+            chain1_num, chain2_num, TMave_mat,
+            seqxA_mat, seqM_mat, seqyA_mat, assign1_list, assign2_list, sequence,
+            d0_scale, 1, 0, 5, ter_opt, split_opt,
+            0, 0, true, true, mirror_opt, resi_vec1, resi_vec2);
+
+
+        /* extract centroid coordinates */
+        double **xcentroids;
+        double **ycentroids;
+        NewArray(&xcentroids, chain1_num, 3);
+        NewArray(&ycentroids, chain2_num, 3);
+        double d0MM=getmin(
+            calculate_centroids(xa_vec, chain1_num, xcentroids),
+            calculate_centroids(ya_vec, chain2_num, ycentroids));
+
+        /* refine enhanced greedy search with centroid superposition */
+        //double het_deg=check_heterooligomer(TMave_mat, chain1_num, chain2_num);
+        homo_refined_greedy_search(TMave_mat, assign1_list,
+            assign2_list, chain1_num, chain2_num, xcentroids,
+            ycentroids, d0MM, len_aa+len_na, ut_mat);
+
+        hetero_refined_greedy_search(TMave_mat, assign1_list,
+            assign2_list, chain1_num, chain2_num, xcentroids,
+            ycentroids, d0MM, len_aa+len_na);
+
+        /* clean up */
+        DeleteArray(&xcentroids, chain1_num);
+        DeleteArray(&ycentroids, chain2_num);
+    }
 
     /* sometime MMalign_iter is even worse than monomer alignment */
     if (byresi_opt==0 && max_total_score<maxTMmono)
@@ -922,7 +968,7 @@ MMAlignResult WrappedMMAlign(const std::vector<geom::Vec3List>& pos_one,
             secx_vec, secy_vec, mol_vec1, mol_vec2, xlen_vec, ylen_vec,
             xa, ya, seqx, seqy, secx, secy, len_aa, len_na, chain1_num, chain2_num,
             TMave_mat, seqxA_mat, seqyA_mat, assign1_list, assign2_list, sequence,
-            d0_scale, fast_opt);
+            d0_scale, fast_opt, chainmap);
     }
 
     /* perform cross chain alignment
@@ -997,8 +1043,7 @@ MMAlignResult WrappedMMAlign(const std::vector<geom::Vec3List>& pos_one,
     ylen_vec.clear();       // length of complex2
     vector<string> ().swap(resi_vec1);  // residue index for chain1
     vector<string> ().swap(resi_vec2);  // residue index for chain2
-
-
+    map<int,int> ().swap(chainmap);
   return res;
 }
 
@@ -1075,7 +1120,8 @@ TMAlignResult WrappedTMAlign(const ost::mol::ChainView& chain1,
 
 MMAlignResult WrappedMMAlign(const ost::mol::EntityView& ent1,
                              const ost::mol::EntityView& ent2,
-                             bool fast) {
+                             bool fast,
+                             const std::map<String, String>& mapping) {
   ost::mol::ChainViewList chains1 = ent1.GetChainList();
   int n1 = chains1.size();
   std::vector<geom::Vec3List> pos1(n1);
@@ -1102,7 +1148,70 @@ MMAlignResult WrappedMMAlign(const ost::mol::EntityView& ent1,
     s2.AddSequence(s);
   }
 
-  return WrappedMMAlign(pos1, pos2, s1, s2, rna1, rna2, fast);
+  std::map<int,int> int_mapping;
+  if(!mapping.empty()) {
+    // OpenStructure mapping is a dict with target chains as key and
+    // model chains as values. Target is ent2 and model is ent1.
+    // USalign needs the mapping the other way around,
+    // i.e. the chain idx of ent1 as key and chain idx of ent2 as value
+    std::set<int> mapped_trg_chains;
+    std::set<int> mapped_mdl_chains;
+    for(auto it = mapping.begin(); it != mapping.end(); ++it) {
+      String trg_cname = it->first;
+      String mdl_cname = it->second;
+      int trg_idx = -1;
+      int mdl_idx = -1;
+
+      for(int i = 0; i < n1; ++i) {
+        if(s1[i].GetName() == mdl_cname) {
+          mdl_idx = i;
+          break;
+        }
+      }
+
+      for(int i = 0; i < n2; ++i) {
+        if(s2[i].GetName() == trg_cname) {
+          trg_idx = i;
+          break;
+        }
+      }
+
+      if(mdl_idx == -1) {
+        std::stringstream ss;
+        ss << "Specified \""<<mdl_cname<<"\" as mdl chain name in custom ";
+        ss << "mapping. Mdl has no mappable chain with this name.";
+        throw ost::Error(ss.str());
+      }
+
+      if(mapped_mdl_chains.find(mdl_idx) != mapped_mdl_chains.end()) {
+        std::stringstream ss;
+        ss << "\""<<mdl_cname<<"\" appears more than once as mdl chain in ";
+        ss << "custom mapping";
+        throw ost::Error(ss.str());
+      }
+
+      if(trg_idx == -1) {
+        std::stringstream ss;
+        ss << "Specified \""<<trg_cname<<"\" as trg chain name in custom ";
+        ss << "mapping. Trg has no mappable chain with this name.";
+        throw ost::Error(ss.str());
+      }
+
+      if(mapped_trg_chains.find(trg_idx) != mapped_trg_chains.end()) {
+        std::stringstream ss;
+        ss << "\""<<trg_cname<<"\" appears more than once as trg chain in ";
+        ss << "custom mapping";
+        throw ost::Error(ss.str());
+      }
+
+      int_mapping[mdl_idx] = trg_idx;
+      mapped_trg_chains.insert(trg_idx);
+      mapped_mdl_chains.insert(mdl_idx);
+    }
+  }
+
+  return WrappedMMAlign(pos1, pos2, s1, s2, rna1, rna2, fast,
+                        int_mapping);
 }
 
 }} //ns
