@@ -29,6 +29,10 @@ class QSEntity:
         self._sequence = dict()
         self._pos = dict()
         self._pair_dist = dict()
+        # min and max xyz for elements in pos used for fast collision
+        # detection
+        self._min_pos = dict()
+        self._max_pos = dict()
 
     @property
     def view(self):
@@ -69,11 +73,12 @@ class QSEntity:
 
         :type: :class:`list` of :class:`tuples`
         """
-        if self._interacting_chains is None:
+        if self._interacting_chains is None:            
             self._interacting_chains = list()
             for x in itertools.combinations(self.chain_names, 2):
-                if np.count_nonzero(self.PairDist(x[0], x[1]) < self.contact_d):
-                    self._interacting_chains.append(x)
+                if self.PotentialInteraction(x[0], x[1]):
+                    if np.count_nonzero(self.PairDist(x[0], x[1]) < self.contact_d):
+                        self._interacting_chains.append(x)
         return self._interacting_chains
     
     def GetChain(self, chain_name):
@@ -134,6 +139,52 @@ class QSEntity:
                                                   'euclidean')
         return self._pair_dist[key]
 
+    def GetMinPos(self, chain_name):
+        """ Get min x,y,z cooridnates for given chain
+
+        Based on positions that are extracted with GetPos
+
+        :param chain_name: Chain in :attr:`~view`
+        :type chain_name: :class:`str`
+        """
+        if chain_name not in self._min_pos:
+            self._min_pos[chain_name] = self.GetPos(chain_name).min(0)
+        return self._min_pos[chain_name]
+
+    def GetMaxPos(self, chain_name):
+        """ Get max x,y,z cooridnates for given chain
+
+        Based on positions that are extracted with GetPos
+
+        :param chain_name: Chain in :attr:`~view`
+        :type chain_name: :class:`str`
+        """
+        if chain_name not in self._max_pos:
+            self._max_pos[chain_name] = self.GetPos(chain_name).max(0)
+        return self._max_pos[chain_name]
+
+    def PotentialInteraction(self, chain_name_one, chain_name_two):
+        """ Returns True if chains potentially interact
+
+        Based on crude collision detection. There is no guarantee
+        that they actually interact if True is returned. However,
+        if False is returned, they don't interact for sure.
+
+        :param chain_name_one: Chain in :attr:`~view`
+        :type chain_name_one: class:`str`
+        :param chain_name_two: Chain in :attr:`~view`
+        :type chain_name_two: class:`str`
+        """
+        min_one = self.GetMinPos(chain_name_one)
+        max_one = self.GetMaxPos(chain_name_one)
+        min_two = self.GetMinPos(chain_name_two)
+        max_two = self.GetMaxPos(chain_name_two)
+        if np.max(min_one - max_two) > self.contact_d:
+            return False
+        if np.max(min_two - max_one) > self.contact_d:
+            return False
+        return True
+
 class QSScorerResult:
     """
     Holds data relevant for QS-score computation. Formulas for QS scores:
@@ -162,11 +213,12 @@ class QSScorerResult:
       from `Xu et al. 2009 <https://dx.doi.org/10.1016%2Fj.jmb.2008.06.002>`_.
     """
     def __init__(self, weighted_scores, weight_sum, weight_extra_mapped,
-                 weight_extra_all):
+                 weight_extra_all, complete_mapping):
         self._weighted_scores = weighted_scores
         self._weight_sum = weight_sum
         self._weight_extra_mapped = weight_extra_mapped
         self._weight_extra_all = weight_extra_all
+        self._complete_mapping = complete_mapping
 
     @property
     def weighted_scores(self):
@@ -200,16 +252,39 @@ class QSScorerResult:
         """
         return self._weight_extra_all
 
+    @property
+    def complete_mapping(self):
+        """ Whether the underlying mapping of the scored assemblies is complete
+
+        In other words: If they have the same stoichiometry. This is relevant
+        for :attr:`~QS_best` and :attr:`~QS_global` in case of no contacts in
+        any of the scored entities.
+
+        :type: :class:`bool`
+        """
+        return self._complete_mapping
+
     @property
     def QS_best(self):
         """ QS_best - the actual score as described in formula section above
 
+        If there are no contacts observed in any of the scored entities this
+        score is 1.0 if we're comparing structures with
+        :attr:`~complete_mapping`, 0.0 otherwise. In the example of two
+        monomers, no contacts can be observed but they exactly match in terms
+        of quaternary structure. Thus a perfect score. In terms of higher order
+        structure that becomes a bit more abstract but in principle they still
+        have the exact same quaternary structure if they match in stoichiometry
+        but have no single contact.
+
         :type: :class:`float`
         """
         nominator = self.weighted_scores
         denominator = self.weight_sum + self.weight_extra_mapped
         if denominator != 0.0:
             return nominator/denominator
+        elif self.complete_mapping:
+            return 1.0
         else:
             return 0.0
 
@@ -217,12 +292,23 @@ class QSScorerResult:
     def QS_global(self):
         """ QS_global - the actual score as described in formula section above
 
+        If there are no contacts observed in any of the scored entities this
+        score is 1.0 if we're comparing structures with
+        :attr:`~complete_mapping`, 0.0 otherwise. In the example of two
+        monomers, no contacts can be observed but they exactly match in terms
+        of quaternary structure. Thus a perfect score. In terms of higher order
+        structure that becomes a bit more abstract but in principle they still
+        have the exact same quaternary structure if they match in stoichiometry
+        but have no single contact.
+
         :type: :class:`float`
         """
         nominator = self.weighted_scores
         denominator = self.weight_sum + self.weight_extra_all
         if denominator != 0.0:
             return nominator/denominator
+        elif self.complete_mapping:
+            return 1.0
         else:
             return 0.0
 
@@ -402,6 +488,11 @@ class QSScorer:
         This only works for interfaces that are computed in :func:`Score`, i.e.
         interfaces for which the alignments are set up correctly.
 
+        As all specified chains must be present, the mapping is considered
+        complete which affects
+        :attr:`QSScorerResult.QS_global`/:attr:`QSScorerResult.QS_best` in
+        edge cases of no observed contacts.
+
         :param trg_ch1: Name of first interface chain in target
         :type trg_ch1: :class:`str`
         :param trg_ch2: Name of second interface chain in target
@@ -429,7 +520,10 @@ class QSScorer:
         trg_int = (trg_ch1, trg_ch2)
         mdl_int = (mdl_ch1, mdl_ch2)
         a, b, c, d = self._MappedInterfaceScores(trg_int, mdl_int)
-        return QSScorerResult(a, b, c, d)
+
+        # complete_mapping is True by definition, as the requested chain pairs
+        # are both present
+        return QSScorerResult(a, b, c, d, True)
 
     def FromFlatMapping(self, flat_mapping):
         """ Same as :func:`Score` but with flat mapping
@@ -475,8 +569,16 @@ class QSScorer:
                 else:
                     weight_extra_all += self._InterfacePenalty2(int2)
 
+        trg_chains = sorted(self.qsent1.chain_names) # should be sorted already
+        mdl_chains = sorted(self.qsent2.chain_names) # should be sorted already
+        mapped_trg_chains = sorted(flat_mapping.keys())
+        mapped_mdl_chains = sorted(flat_mapping.values())
+        trg_complete = trg_chains == mapped_trg_chains
+        mdl_complete = mdl_chains == mapped_mdl_chains
+        complete_mapping = trg_complete and mdl_complete
+
         return QSScorerResult(weighted_scores, weight_sum, weight_extra_mapped,
-                              weight_extra_all)
+                              weight_extra_all, complete_mapping)
 
     def _MappedInterfaceScores(self, int1, int2):
         key_one = (int1, int2)
@@ -518,22 +620,50 @@ class QSScorer:
         contact_d = self.qsent1.contact_d
         mapped_idx_grid_1 = np.ix_(mapped_indices_1_1, mapped_indices_2_1)
         mapped_idx_grid_2 = np.ix_(mapped_indices_1_2, mapped_indices_2_2)
-        mapped_d1_contacts = d1[mapped_idx_grid_1] < contact_d
-        mapped_d2_contacts = d2[mapped_idx_grid_2] < contact_d
-        assert(mapped_d1_contacts.shape == mapped_d2_contacts.shape)
-        shared_mask = np.logical_and(mapped_d1_contacts, mapped_d2_contacts)
-        shared_mask_d1 = np.full(d1.shape, False, dtype=bool)
-        shared_mask_d1[mapped_idx_grid_1] = shared_mask
-        shared_mask_d2 = np.full(d2.shape, False, dtype=bool)
-        shared_mask_d2[mapped_idx_grid_2] = shared_mask
 
-        # get mapped but nonshared masks
-        mapped_nonshared_mask_d1 = np.full(d1.shape, False, dtype=bool)
-        mapped_nonshared_mask_d1[mapped_idx_grid_1] = \
-        np.logical_and(np.logical_not(shared_mask), mapped_d1_contacts)
-        mapped_nonshared_mask_d2 = np.full(d2.shape, False, dtype=bool)
-        mapped_nonshared_mask_d2[mapped_idx_grid_2] = \
-        np.logical_and(np.logical_not(shared_mask), mapped_d2_contacts)
+        if mapped_indices_1_1.shape[0] == 0 or mapped_indices_2_1.shape[0] == 0:
+            # dealing with special cases where we have no mapped residues
+            # we only avoid errors here when using maped_idx_grid_x for indexing
+            # but run the rest of the algorithm anyways which produces some
+            # computational overhead. Thats OK, as this should occur rarely
+            shared_mask_d1 = np.full(d1.shape, False, dtype=bool)
+            shared_mask_d2 = np.full(d2.shape, False, dtype=bool)
+            mapped_nonshared_mask_d1 = np.full(d1.shape, False, dtype=bool)
+            mapped_nonshared_mask_d2 = np.full(d2.shape, False, dtype=bool)
+            if mapped_indices_1_1.shape[0] == 0 or \
+               mapped_indices_2_1.shape[0] == 0:
+                # mapped_idx_grid_1 has not a single mapped residue which raises
+                # an error when calling something like d1[mapped_idx_grid_1]
+                mapped_d1_contacts = np.full(d1.shape, False, dtype=bool)
+            else:
+                mapped_d1_contacts = d1[mapped_idx_grid_1] < contact_d
+                mapped_nonshared_mask_d1[mapped_idx_grid_1] = mapped_d1_contacts
+
+            if mapped_indices_1_2.shape[0] == 0 or \
+               mapped_indices_2_2.shape[0] == 0:
+                # mapped_idx_grid_2 has not a single mapped residue which raises
+                # an error when calling something like d2[mapped_idx_grid_2]
+                mapped_d2_contacts = np.full(d2.shape, False, dtype=bool)
+            else:
+                mapped_d2_contacts = d2[mapped_idx_grid_2] < contact_d
+                mapped_nonshared_mask_d2[mapped_idx_grid_2] = mapped_d2_contacts
+            shared_mask = np.full(mapped_d1_contacts.shape, False, dtype=bool)
+        else:
+            mapped_d1_contacts = d1[mapped_idx_grid_1] < contact_d
+            mapped_d2_contacts = d2[mapped_idx_grid_2] < contact_d
+            shared_mask = np.logical_and(mapped_d1_contacts, mapped_d2_contacts)
+            shared_mask_d1 = np.full(d1.shape, False, dtype=bool)
+            shared_mask_d1[mapped_idx_grid_1] = shared_mask
+            shared_mask_d2 = np.full(d2.shape, False, dtype=bool)
+            shared_mask_d2[mapped_idx_grid_2] = shared_mask
+
+            # get mapped but nonshared masks
+            mapped_nonshared_mask_d1 = np.full(d1.shape, False, dtype=bool)
+            mapped_nonshared_mask_d1[mapped_idx_grid_1] = \
+            np.logical_and(np.logical_not(shared_mask), mapped_d1_contacts)
+            mapped_nonshared_mask_d2 = np.full(d2.shape, False, dtype=bool)
+            mapped_nonshared_mask_d2[mapped_idx_grid_2] = \
+            np.logical_and(np.logical_not(shared_mask), mapped_d2_contacts)
 
         # contributions from shared contacts
         shared_d1 = d1[shared_mask_d1]
@@ -623,4 +753,4 @@ class QSScorer:
         return penalty
 
 # specify public interface
-__all__ = ('QSEntity', 'QSScorer')
+__all__ = ('QSEntity', 'QSScorer', 'QSScorerResult')
